@@ -35,7 +35,6 @@ GtkWidget *canvas;
 
 char *GdmGraphicalTheme = NULL;
 char *GdmGraphicalThemeDir = NULL;
-char *GdmDefaultLocale = NULL;
 int GdmXineramaScreen = 0;
 gboolean GdmShowGnomeChooserSession = FALSE;
 gboolean GdmShowGnomeFailsafeSession = FALSE;
@@ -91,7 +90,6 @@ greeter_parse_config (void)
         g_free (GdmGraphicalThemeDir);
         GdmGraphicalThemeDir = g_strdup (GREETERTHEMEDIR);
       }
-    GdmDefaultLocale = ve_config_get_string (config, GDM_KEY_LOCALE);
     GdmXineramaScreen = ve_config_get_int (config, GDM_KEY_XINERAMASCREEN);
     GdmUseCirclesInEntry = ve_config_get_bool (config, GDM_KEY_ENTRY_CIRCLES);
 
@@ -819,9 +817,6 @@ greeter_reread_config (int sig, gpointer data)
 	/* Also we may not need to check ALL those keys but just a few */
 	if (strcmp (theme, GdmGraphicalTheme) != 0 ||
 	    strcmp (theme_dir, GdmGraphicalThemeDir) != 0 ||
-	     ! string_same (config,
-			    GdmDefaultLocale,
-			    GDM_KEY_LOCALE) ||
 	     ! string_same (config,
 			    GdmGtkRC,
 			    GDM_KEY_GTKRC) ||
@@ -919,11 +914,18 @@ get_theme_file (const char *in, char **theme_dir)
       dir = NULL;
       if (DOING_GDM_DEVELOPMENT)
         {
-          dir = g_build_filename ("themes", in, NULL);
-	  if (access (dir, F_OK) != 0)
+	  if (access (in, F_OK) == 0)
 	    {
-	      g_free (dir);
-	      dir = NULL;
+	      dir = g_strdup (in);
+	    }
+	  else
+	    {
+              dir = g_build_filename ("themes", in, NULL);
+	      if (access (dir, F_OK) != 0)
+	        {
+	          g_free (dir);
+	          dir = NULL;
+	        }
 	    }
 	}
       if (dir == NULL)
@@ -939,10 +941,12 @@ get_theme_file (const char *in, char **theme_dir)
   }
   if (access (info, F_OK) != 0)
     {
+      char *base = g_path_get_basename (in);
       /* just guess the name, we have no info about the theme at
        * this point */
       g_free (info);
-      file = g_strdup_printf ("%s/%s.xml", dir, in);
+      file = g_strdup_printf ("%s/%s.xml", dir, base);
+      g_free (base);
       return file;
     }
 
@@ -961,6 +965,42 @@ get_theme_file (const char *in, char **theme_dir)
   return file;
 }
 
+/* Not to look too shaby on Xinerama setups */
+static void
+setup_background_color (void)
+{
+  GdkColormap *colormap;
+  GdkColor color;
+  VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+  char *bg_color = ve_config_get_string (config, GDM_KEY_BACKGROUNDCOLOR);
+
+  if (bg_color == NULL ||
+      bg_color[0] == '\0' ||
+      ! gdk_color_parse (bg_color, &color))
+    {
+      gdk_color_parse ("#007777", &color);
+    }
+
+  g_free (bg_color);
+
+  colormap = gdk_drawable_get_colormap
+	  (gdk_get_default_root_window ());
+  /* paranoia */
+  if (colormap != NULL)
+    {
+      gboolean success;
+      gdk_error_trap_push ();
+
+      gdk_colormap_alloc_colors (colormap, &color, 1,
+				 FALSE, TRUE, &success);
+      gdk_window_set_background (gdk_get_default_root_window (), &color);
+      gdk_window_clear (gdk_get_default_root_window ());
+
+      gdk_flush ();
+      gdk_error_trap_pop ();
+    }
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -985,14 +1025,7 @@ main (int argc, char *argv[])
 
   greeter_parse_config ();
 
-  /* no language set, use the GdmDefaultLocale */
-  if (GdmDefaultLocale != NULL &&
-      strlen (GdmDefaultLocale) != 0 &&
-      g_getenv ("LANG") == NULL &&
-      g_getenv ("LC_ALL") == NULL)
-    setlocale (LC_ALL, GdmDefaultLocale);
-  else
-    setlocale (LC_ALL, "");
+  setlocale (LC_ALL, "");
 
   gtk_init (&argc, &argv);
 
@@ -1060,6 +1093,7 @@ main (int argc, char *argv[])
 		      G_CALLBACK (key_press_event), NULL);
   
   canvas = gnome_canvas_new_aa ();
+  GTK_WIDGET_UNSET_FLAGS (canvas, GTK_CAN_FOCUS);
   gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas),
 				  0.0, 0.0,
 				  (double) gdm_wm_screen.width,
@@ -1070,6 +1104,18 @@ main (int argc, char *argv[])
 			       gdm_wm_screen.height);
   gtk_container_add (GTK_CONTAINER (window), canvas);
 
+  if (g_getenv ("GDM_THEME") != NULL)
+    {
+      g_free (GdmGraphicalTheme);
+      GdmGraphicalTheme = g_strdup (g_getenv ("GDM_THEME"));
+    }
+  if (DOING_GDM_DEVELOPMENT &&
+      g_getenv ("GDM_FAKE_TIMED") != NULL)
+    {
+      g_free (GdmTimedLogin);
+      GdmTimedLogin = g_strdup ("fake");
+    }
+
   theme_file = get_theme_file (GdmGraphicalTheme, &theme_dir);
   
   error = NULL;
@@ -1079,6 +1125,33 @@ main (int argc, char *argv[])
 			gdm_wm_screen.height,
 			&error);
 
+    if (root == NULL &&
+	g_getenv ("GDM_THEME") != NULL &&
+	DOING_GDM_DEVELOPMENT)
+      {
+        GtkWidget *dialog;
+
+        gdm_wm_init (0);
+        gdm_wm_focus_new_windows (TRUE);
+    
+        dialog = gtk_message_dialog_new (NULL /* parent */,
+				         GTK_DIALOG_MODAL /* flags */,
+				         GTK_MESSAGE_ERROR,
+				         GTK_BUTTONS_OK,
+				         _("There was an error loading the "
+					   "theme %s"),
+					 g_getenv ("GDM_THEME"));
+    
+        gtk_widget_show_all (dialog);
+        gdm_wm_center_window (GTK_WINDOW (dialog));
+
+        setup_cursor (GDK_LEFT_PTR);
+    
+        gtk_dialog_run (GTK_DIALOG (dialog));
+
+        exit(1);
+      }
+
   /* Try circles.xml */
   if (root == NULL)
     {
@@ -1092,6 +1165,39 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
+  if (greeter_lookup_id ("user-pw-entry") == NULL)
+    {
+      GtkWidget *dialog;
+
+      gdm_wm_init (0);
+      gdm_wm_focus_new_windows (TRUE);
+    
+      dialog = gtk_message_dialog_new (NULL /* parent */,
+				       GTK_DIALOG_MODAL /* flags */,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("The theme for the graphical greeter "
+					 "is corrupt.  It does not contain "
+					 "definition for the username/password "
+					 "entry element."));
+
+      gtk_widget_show_all (dialog);
+      gdm_wm_center_window (GTK_WINDOW (dialog));
+
+      setup_cursor (GDK_LEFT_PTR);
+
+      gtk_dialog_run (GTK_DIALOG (dialog));
+
+      root = NULL;
+    }
+
+  /* FIXME: beter information should be printed */
+  if (DOING_GDM_DEVELOPMENT && root == NULL)
+    {
+      g_warning ("No theme could be loaded");
+      exit(1);
+    }
+
   if (root == NULL)
     {
       GtkWidget *dialog;
@@ -1147,6 +1253,8 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
+  setup_background_color ();
+
   /* can it ever happen that it'd be NULL here ??? */
   if (window->window != NULL)
     {
